Crate aeon_tk

Crate aeon_tk 

Re-exports

pub use aeon_macros as macros;

Modules

array
This module contains several “hacky” utilities to work around the current lack of generic_const_exprs in Rust.
element
geometry
A module for various geometric primatives (AABBs, working with cartesian indices, etc.).
image
kernel
A crate for approximating numerical operators on uniform rectangular meshes using finite differencing.
mesh
Module containing the Mesh API, the main datastructure through which one writes finite difference programs.
prelude
Provides common types used for most aeon applications.
shared
Helpers for working with slices of unsafe cells.
solver
High-level solvers and integrators for common numerical problems.

Structs

IRef
A common helper wrapper for implementing a trait T for types of the form &impl T