pub struct ActiveCellId(pub usize);Expand description
Index into active cells in tree.
This is the primary representation of cells in a Tree, as degrees
of freedom are only assigned to active cells. Can be converted to generic CellIndex via
tree.cell_from_active_index(
Tuple Fields
0: usizeTrait Implementations
Sourceimpl Clone for ActiveCellId
impl Clone for ActiveCellId
Sourcefn clone(&self) -> ActiveCellId
fn clone(&self) -> ActiveCellId
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl DataSize for ActiveCellId
impl DataSize for ActiveCellId
Sourceconst IS_DYNAMIC: bool = <usize as datasize::DataSize>::IS_DYNAMIC
const IS_DYNAMIC: bool = <usize as datasize::DataSize>::IS_DYNAMIC
If
true, the type has a heap size that can vary at runtime, depending on the actual value.Sourceconst STATIC_HEAP_SIZE: usize = <usize as datasize::DataSize>::STATIC_HEAP_SIZE
const STATIC_HEAP_SIZE: usize = <usize as datasize::DataSize>::STATIC_HEAP_SIZE
The amount of space a value of the type always occupies. If
IS_DYNAMIC is false, this is
the total amount of heap memory occupied by the value. Otherwise this is a lower bound.Sourcefn estimate_heap_size(&self) -> usize
fn estimate_heap_size(&self) -> usize
Estimates the size of heap memory taken up by this value. Read more
Sourceimpl Debug for ActiveCellId
impl Debug for ActiveCellId
Sourceimpl<'de> Deserialize<'de> for ActiveCellId
impl<'de> Deserialize<'de> for ActiveCellId
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl Hash for ActiveCellId
impl Hash for ActiveCellId
Sourceimpl Ord for ActiveCellId
impl Ord for ActiveCellId
Sourcefn cmp(&self, other: &ActiveCellId) -> Ordering
fn cmp(&self, other: &ActiveCellId) -> Ordering
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialEq for ActiveCellId
impl PartialEq for ActiveCellId
Sourceimpl PartialOrd for ActiveCellId
impl PartialOrd for ActiveCellId
Sourceimpl Serialize for ActiveCellId
impl Serialize for ActiveCellId
impl Copy for ActiveCellId
impl Eq for ActiveCellId
impl StructuralPartialEq for ActiveCellId