pub struct Tree<const N: usize> { /* private fields */ }Expand description
An N-dimensional hypertree, which subdives each axis in two in
each refinement step.
Used as a basis for axes aligned adaptive finite difference meshes. The tree is
Implementations
Sourceimpl<const N: usize> Tree<N>
impl<const N: usize> Tree<N>
Sourcepub fn new(domain: HyperBox<N>) -> Self
pub fn new(domain: HyperBox<N>) -> Self
Constructs a new tree consisting of a single root cell, covering the given domain.
pub fn set_periodic(&mut self, axis: usize, periodic: bool)
pub fn domain(&self) -> HyperBox<N>
Sourcepub fn num_active_cells(&self) -> usize
pub fn num_active_cells(&self) -> usize
The number of active (leaf) cells in this tree.
Sourcepub fn num_levels(&self) -> usize
pub fn num_levels(&self) -> usize
The maximum depth of this tree.
pub fn level_cells( &self, level: usize, ) -> impl Iterator<Item = CellId> + ExactSizeIterator
pub fn cell_indices(&self) -> impl Iterator<Item = CellId>
pub fn active_cell_indices(&self) -> impl Iterator<Item = ActiveCellId>
Sourcepub fn bounds(&self, cell: CellId) -> HyperBox<N>
pub fn bounds(&self, cell: CellId) -> HyperBox<N>
Returns the numerical bounds of a given cell.
pub fn active_bounds(&self, active: ActiveCellId) -> HyperBox<N>
pub fn active_level(&self, cell: ActiveCellId) -> usize
Sourcepub fn children(&self, cell: CellId) -> Option<CellId>
pub fn children(&self, cell: CellId) -> Option<CellId>
Returns the children of a given node. Node must not be leaf.
Sourcepub fn child(&self, cell: CellId, child: Split<N>) -> Option<CellId>
pub fn child(&self, cell: CellId, child: Split<N>) -> Option<CellId>
Returns a child of a give node.
Sourcepub fn active_zvalue(&self, active: ActiveCellId) -> &BitSlice<usize, Lsb0> ⓘ
pub fn active_zvalue(&self, active: ActiveCellId) -> &BitSlice<usize, Lsb0> ⓘ
Returns the zvalue of the given active cell.
pub fn active_split(&self, active: ActiveCellId, level: usize) -> Split<N>
pub fn most_recent_active_split(&self, active: ActiveCellId) -> Option<Split<N>>
Sourcepub fn check_refine_flags(&self, flags: &[bool]) -> bool
pub fn check_refine_flags(&self, flags: &[bool]) -> bool
Checks whether the given refinement flags are balanced.
Sourcepub fn balance_refine_flags(&self, flags: &mut [bool])
pub fn balance_refine_flags(&self, flags: &mut [bool])
Balances the given refinement flags, flagging additional cells for refinement to preserve the 2:1 fine coarse ratio between every two neighbors.
Sourcepub fn refine_active_index_map(&self, flags: &[bool], map: &mut [ActiveCellId])
pub fn refine_active_index_map(&self, flags: &[bool], map: &mut [ActiveCellId])
Fills the map with updated indices after refinement is performed. If a cell is refined, this will point to the base cell in that new subdivision.
pub fn refine(&mut self, flags: &[bool])
Sourcepub fn check_coarsen_flags(&self, flags: &[bool]) -> bool
pub fn check_coarsen_flags(&self, flags: &[bool]) -> bool
Checks that the given coarsening flags are balanced and valid.
Sourcepub fn balance_coarsen_flags(&self, flags: &mut [bool])
pub fn balance_coarsen_flags(&self, flags: &mut [bool])
Balances the given coarsening flags
Sourcepub fn coarsen_active_index_map(&self, flags: &[bool], map: &mut [ActiveCellId])
pub fn coarsen_active_index_map(&self, flags: &[bool], map: &mut [ActiveCellId])
Maps current cells to indices after coarsening is performed.
pub fn coarsen(&mut self, flags: &[bool])
pub fn build(&mut self)
Sourcepub fn cell_from_active_index(&self, active: ActiveCellId) -> CellId
pub fn cell_from_active_index(&self, active: ActiveCellId) -> CellId
Computes the cell index corresponding to an active cell.
Sourcepub fn active_index_from_cell(&self, cell: CellId) -> Option<ActiveCellId>
pub fn active_index_from_cell(&self, cell: CellId) -> Option<ActiveCellId>
Computes active cell index from a cell, returning None if cell is
not active.
Sourcepub fn active_children(
&self,
cell: CellId,
) -> impl Iterator<Item = ActiveCellId> + ExactSizeIterator
pub fn active_children( &self, cell: CellId, ) -> impl Iterator<Item = ActiveCellId> + ExactSizeIterator
Returns an iterator over active cells that are children of the given cell.
If is_active(cell) = true then this iterator will be a singleton
returning the same value as tree.active_index_from_cell(cell).
Sourcepub fn cell_from_point(&self, point: [f64; N]) -> CellId
pub fn cell_from_point(&self, point: [f64; N]) -> CellId
Returns the cell which owns the given point. Performs in O(log N).
Sourcepub fn cell_from_point_cached(&self, point: [f64; N], cache: CellId) -> CellId
pub fn cell_from_point_cached(&self, point: [f64; N], cache: CellId) -> CellId
Returns the node which owns the given point, shortening this search with an initial guess. Rather than operating in O(log N) time, this approaches O(1) if the guess is sufficiently close.
Sourcepub fn neighbor(&self, cell: CellId, face: Face<N>) -> Option<CellId>
pub fn neighbor(&self, cell: CellId, face: Face<N>) -> Option<CellId>
Returns the neighboring cell along the given face. If the neighboring cell is more refined, this
returns the cell index of the adjacent cell with tree.level(neighbor) == tree.level(cell).
If this passes over a nonperiodic boundary then it returns None.
Sourcepub fn neighbor_region(&self, cell: CellId, region: Region<N>) -> Option<CellId>
pub fn neighbor_region(&self, cell: CellId, region: Region<N>) -> Option<CellId>
Returns the neighboring cell in the given region. If the neighboring cell is more refined, this
returns the cell index of the adjacent cell with tree.level(neighbor) == tree.level(cell).
If this passes over a nonperiodic boundary then it returns None.
Sourcepub fn _neighbor_region2(
&self,
cell: CellId,
region: Region<N>,
) -> Option<CellId>
pub fn _neighbor_region2( &self, cell: CellId, region: Region<N>, ) -> Option<CellId>
Returns the neighboring cell in the given region. If the neighboring cell is more refined, this
returns the cell index of the adjacent cell with tree.level(neighbor) == tree.level(cell).
If this passes over a nonperiodic boundary then it returns None.
Sourcepub fn active_neighbors_in_region(
&self,
cell: CellId,
region: Region<N>,
) -> impl Iterator<Item = ActiveCellId> + '_
pub fn active_neighbors_in_region( &self, cell: CellId, region: Region<N>, ) -> impl Iterator<Item = ActiveCellId> + '_
Iterates over
pub fn active_neighborhood( &self, cell: ActiveCellId, ) -> impl Iterator<Item = ActiveCellId> + '_
pub fn active_coarse_neighborhood( &self, cell: ActiveCellId, ) -> impl Iterator<Item = ActiveCellId> + '_
Sourcepub fn is_boundary_face(&self, cell: CellId, face: Face<N>) -> bool
pub fn is_boundary_face(&self, cell: CellId, face: Face<N>) -> bool
Returns true if a face lies on a boundary.
Sourcepub fn boundary_region(&self, cell: CellId, region: Region<N>) -> Region<N>
pub fn boundary_region(&self, cell: CellId, region: Region<N>) -> Region<N>
Given a neighboring region to a cell, determines which global region that belongs to (usually)