pub struct Image { /* private fields */ }Expand description
Several fields of data spread among
Implementations
Sourceimpl Image
impl Image
Sourcepub fn new(channels: usize, nodes: usize) -> Self
pub fn new(channels: usize, nodes: usize) -> Self
Allocates a new image with the specified number of channels and nodes
pub fn reinit(&mut self, channels: usize, nodes: usize)
pub fn resize(&mut self, nodes: usize)
pub fn num_nodes(&self) -> usize
pub fn is_empty(&self) -> bool
Sourcepub fn from_storage(data: Vec<f64>, channels: usize) -> Self
pub fn from_storage(data: Vec<f64>, channels: usize) -> Self
Constructs a new system vector from the given data. data.len() must be divisible by field_count::<Label>().
Sourcepub fn into_storage(self) -> Vec<f64>
pub fn into_storage(self) -> Vec<f64>
Transforms a system vector back into a linear vector
pub fn storage(&self) -> &[f64]
pub fn storage_mut(&mut self) -> &mut [f64]
pub fn num_channels(&self) -> usize
pub fn channels(&self) -> Range<usize>
pub fn channel(&self, channel: usize) -> &[f64]
pub fn channel_mut(&mut self, channel: usize) -> &mut [f64]
pub fn as_ref(&self) -> ImageRef<'_>
pub fn as_mut(&mut self) -> ImageMut<'_>
pub fn slice<R>(&self, range: R) -> ImageRef<'_>
pub fn slice_mut<R>(&mut self, range: R) -> ImageMut<'_>