Module kernel

Module kernel 

A crate for approximating numerical operators on uniform rectangular meshes using finite differencing.

Structs

Derivative
Derivative operation of a given order.
DirichletParams
Dissipation
Kriss Olgier dissipation of the given order.
Element
A reference element defined on [-1, 1]^N. This element implements code for wavelet transformations and general operator approximation, whereas NodeSpace implements a much stricter subset of operations with fixed precomputed weights.
EmptyConditions
Gradient
Computes the gradient along the given axis.
Hessian
Computes the mixed derivative of the given axes.
Interpolation
NodeCartesianIter
A helper for iterating over a node window.
NodePlaneIter
A helper for iterating over a plane in a node window.
NodeSpace
A uniform rectangular domain of nodes to which various derivative and interpolations can be applied.
NodeWindow
Defines a rectagular region of a larger NodeSpace.
RadiativeParams
Describes a radiative boundary condition at a point on the boundary.
ScalarConditions
Transforms a single condition into a set of Conditions<N> where Self::System = Scalar.
SecondDerivative
Second derivative operator of a given order.
Unimplemented
Unimplemented Kernel (used for debugging)
Value
Value operation.

Enums

Border
BoundaryClass
BoundaryKind
Indicates what type of boundary condition is used along a particualr face of the domain. More specific boundary conditions are provided by the Condition API, but for many funtions, Boundary provides enough information to compute supports and apply stencils.

Traits

BoundaryConds
Provides specifics for enforcing boundary conditions for a particular field.
Convolution
A N-dimensional tensor product of several seperable kernels.
Interpolant
Kernel
SystemBoundaryConds
A generalization of Condition<N> for a coupled systems of scalar fields.

Functions

is_boundary_compatible
Checks whether a set of boundary conditions are compatible with the given ghost flags.
node_from_vertex
Converts a (unsigned) vertex index into an (signed) node index.
vertex_from_node
Converts a (signed) node index into an (unsigned) vertex index.