pub enum BoundaryKind {
Symmetric,
AntiSymmetric,
Custom,
Radiative,
Free,
StrongDirichlet,
WeakDirichlet,
}Expand description
Indicates what type of boundary condition is used along a particualr
face of the domain. More specific boundary conditions are provided
by the Condition API, but for many funtions, Boundary provides
enough information to compute supports and apply stencils.
Variants
Symmetric
Symmetric Boundary condition. Function is even along axis (so function values are reflected across the axis with the same sign)
AntiSymmetric
Antisymmetric Boundary condition. Function is even along axis (so function values are reflected across the axis with the opposite sign, on axis set to zero)
Custom
This boundary condition indicates that the ghost nodes have been filled manually via some external process. This can be used to implement custom boundary conditions, and is primarily used to fill inter-grid boundaries in the adaptive mesh refinement driver.
Radiative
The boundary condition is implemented via
Free
StrongDirichlet
Boundary is set to a given value, lopsided stencils are used near the boundary. This condition can either be strongly enforced (values of systems are set at boundary before application) or weakly enforced (condition is applied to time derivative of system).
WeakDirichlet
Implementations
Sourceimpl BoundaryKind
impl BoundaryKind
Sourcepub fn needs_ghost(self) -> bool
pub fn needs_ghost(self) -> bool
Are ghost nodes are used when enforcing this kind of boundary condition?
pub fn class(self) -> BoundaryClass