pub struct Element<const N: usize> { /* private fields */ }Expand description
A reference element defined on [-1, 1]^N. This element implements code
for wavelet transformations and general operator approximation, whereas
NodeSpace implements a much stricter subset of operations with fixed
precomputed weights.
Implementations
Sourceimpl<const N: usize> Element<N>
impl<const N: usize> Element<N>
Sourcepub fn uniform(width: usize, order: usize) -> Self
pub fn uniform(width: usize, order: usize) -> Self
Constructs a reference element with uniformly placed
support points with width + 1 points along each axis.
Sourcepub fn support_refined(&self) -> usize
pub fn support_refined(&self) -> usize
Number of supports points after the element has been refined.
pub fn prolong_stencil(&self, target: usize) -> ColRef<'_, f64>
Sourcepub fn position(&self, index: [usize; N]) -> [f64; N]
pub fn position(&self, index: [usize; N]) -> [f64; N]
Iterates the position of a support point in the element.
pub fn position_refined(&self, index: [usize; N]) -> [f64; N]
pub fn space(&self) -> IndexSpace<N>
pub fn space_refined(&self) -> IndexSpace<N>
Sourcepub fn nodal_indices(&self) -> impl Iterator<Item = [usize; N]>
pub fn nodal_indices(&self) -> impl Iterator<Item = [usize; N]>
Iterates over all nodal coefficients in a wavelet representation on this element.
Sourcepub fn diagonal_indices(&self) -> impl Iterator<Item = [usize; N]>
pub fn diagonal_indices(&self) -> impl Iterator<Item = [usize; N]>
Iterates over all diagonal detail coefficients in a wavelet representation on this element.
Sourcepub fn diagonal_int_indices(
&self,
buffer: usize,
) -> impl Iterator<Item = [usize; N]>
pub fn diagonal_int_indices( &self, buffer: usize, ) -> impl Iterator<Item = [usize; N]>
Iterates over diagonal detail coefficients in a wavelet representation of this element,
ignoring elements within a buffer around the edge of the refined support.
Sourcepub fn detail_indices(&self) -> impl Iterator<Item = [usize; N]>
pub fn detail_indices(&self) -> impl Iterator<Item = [usize; N]>
Iterates over all detail coefficients in a wavelet representation on this element.
pub fn nodal_points(&self) -> impl Iterator<Item = usize>
pub fn diagonal_points(&self) -> impl Iterator<Item = usize>
pub fn diagonal_int_points(&self, buffer: usize) -> impl Iterator<Item = usize>
pub fn detail_points(&self) -> impl Iterator<Item = usize>
Sourcepub fn prolong(&self, source: &[f64], dest: &mut [f64])
pub fn prolong(&self, source: &[f64], dest: &mut [f64])
Prolongs data from the element to a refined version of the element.
Sourcepub fn prolong_in_place(&self, dest: &mut [f64])
pub fn prolong_in_place(&self, dest: &mut [f64])
Fills in-between points on dest using interpolation, assuming that nodal points on dest have been properly filled.
Sourcepub fn inject(&self, source: &[f64], dest: &mut [f64])
pub fn inject(&self, source: &[f64], dest: &mut [f64])
Performs injection from source to a refined dest.
Sourcepub fn restrict(&self, source: &[f64], dest: &mut [f64])
pub fn restrict(&self, source: &[f64], dest: &mut [f64])
Restricts refined data from source onto dest.
Sourcepub fn wavelet(&self, source: &[f64], dest: &mut [f64])
pub fn wavelet(&self, source: &[f64], dest: &mut [f64])
Computes the wavelet coefficients for the given function.
Sourcepub fn wavelet_rel_error(&self, coefs: &[f64]) -> f64
pub fn wavelet_rel_error(&self, coefs: &[f64]) -> f64
Computes the relative error between the wavelet’s representation and a nodal approximation.
Sourcepub fn wavelet_abs_error(&self, coefs: &[f64]) -> f64
pub fn wavelet_abs_error(&self, coefs: &[f64]) -> f64
Computes the absolute error between the wavelet’s representation and a nodal approximation.