pub struct NodeSpace<const N: usize> {
pub size: [usize; N],
pub ghost: usize,
pub bounds: HyperBox<N>,
pub boundary: FaceArray<N, BoundaryClass>,
}Expand description
A uniform rectangular domain of nodes to which various derivative and interpolations can be applied.
Fields
size: [usize; N]Number of cells along each axis (one less than then number of vertices).
ghost: usizeNumber of ghost nodes along each face.
bounds: HyperBox<N>Position of the node space.
boundary: FaceArray<N, BoundaryClass>Implementations
Sourceimpl<const N: usize> NodeSpace<N>
impl<const N: usize> NodeSpace<N>
pub fn ghost(&self) -> usize
pub fn cell_size(&self) -> [usize; N]
pub fn vertex_size(&self) -> [usize; N]
pub fn node_size(&self) -> [usize; N]
pub fn spacing_axis(&self, axis: usize) -> f64
Sourcepub fn is_interior(&self, node: [isize; N]) -> bool
pub fn is_interior(&self, node: [isize; N]) -> bool
Returns true if the node lies inside the interior of the nodespace (i.e. it is not a ghost node).
Sourcepub fn index_from_node(&self, node: [isize; N]) -> usize
pub fn index_from_node(&self, node: [isize; N]) -> usize
Computes a linear index from a nodal index.
Sourcepub fn index_from_vertex(&self, vertex: [usize; N]) -> usize
pub fn index_from_vertex(&self, vertex: [usize; N]) -> usize
Computes a linear index from a vertex index.
Sourcepub fn inner_window(&self) -> NodeWindow<N>
pub fn inner_window(&self) -> NodeWindow<N>
Returns a window of just the interior and edges of the node space (no ghost nodes).
Sourcepub fn full_window(&self) -> NodeWindow<N>
pub fn full_window(&self) -> NodeWindow<N>
Returns the window which encompasses the whole node space
Sourcepub fn region_window(&self, extent: usize, region: Region<N>) -> NodeWindow<N>
pub fn region_window(&self, extent: usize, region: Region<N>) -> NodeWindow<N>
Returns a window over a region of the node space.
Sourcepub fn face_window(&self, _: Face<N>) -> NodeWindow<N>
pub fn face_window(&self, _: Face<N>) -> NodeWindow<N>
Returns a window over all nodes on a given face.
pub fn face_window_disjoint(&self, face: Face<N>) -> NodeWindow<N>
pub fn contains_window(&self, window: NodeWindow<N>) -> bool
Sourcepub fn fill_boundary(
&self,
extent: usize,
cond: impl BoundaryConds<N>,
dest: &mut [f64],
)
pub fn fill_boundary( &self, extent: usize, cond: impl BoundaryConds<N>, dest: &mut [f64], )
Set strongly enforced boundary conditions. This enforces parity and dirichlet boundary
conditions on this particular NodeSpace.
Sourcepub fn apply(
&self,
corner: [isize; N],
stencils: [&[f64]; N],
field: &[f64],
) -> f64
pub fn apply( &self, corner: [isize; N], stencils: [&[f64]; N], field: &[f64], ) -> f64
Applies a tensor product of stencils to a given field.
Sourcepub fn apply_axis(
&self,
corner: [isize; N],
stencil: &[f64],
field: &[f64],
axis: usize,
) -> f64
pub fn apply_axis( &self, corner: [isize; N], stencil: &[f64], field: &[f64], axis: usize, ) -> f64
Applies a single stencil along an axis to a given field.
pub fn evaluate_interior( &self, convolution: impl Convolution<N>, node: [isize; N], field: &[f64], ) -> f64
pub fn evaluate_axis_interior( &self, kernel: impl Kernel, node: [isize; N], field: &[f64], axis: usize, ) -> f64
Sourcepub fn evaluate(
&self,
convolution: impl Convolution<N>,
node: [isize; N],
field: &[f64],
) -> f64
pub fn evaluate( &self, convolution: impl Convolution<N>, node: [isize; N], field: &[f64], ) -> f64
Evaluates the operation of a convolution at a given node.