pub struct Checkpoint<const N: usize> { /* private fields */ }Expand description
Represents a snapshot of a Mesh along with relavent field data.
The Checkpoint<N> can then be serialized and deserialized from disk,
allowing data to be loaded/reused across runs.
Implementations
Sourceimpl<const N: usize> Checkpoint<N>
impl<const N: usize> Checkpoint<N>
Sourcepub fn attach_mesh(&mut self, mesh: &Mesh<N>)
pub fn attach_mesh(&mut self, mesh: &Mesh<N>)
Attaches a mesh to the checkpoint
Sourcepub fn set_embedding<const S: usize>(&mut self, positions: &[[f64; S]])
pub fn set_embedding<const S: usize>(&mut self, positions: &[[f64; S]])
Sets the mesh as embedded in a higher dimensional space.
Sourcepub fn save_image(&mut self, name: &str, data: ImageRef<'_>)
pub fn save_image(&mut self, name: &str, data: ImageRef<'_>)
Saves image data with the given name key.
Sourcepub fn read_image(&self, name: &str) -> Image
pub fn read_image(&self, name: &str) -> Image
Reads image data associated to the given name.
Sourcepub fn save_field(&mut self, name: &str, data: &[f64])
pub fn save_field(&mut self, name: &str, data: &[f64])
Attaches a field for serialization in the model.
Sourcepub fn load_field(&self, name: &str, data: &mut Vec<f64>)
pub fn load_field(&self, name: &str, data: &mut Vec<f64>)
Reads a field from the model.
Sourcepub fn read_field(&self, name: &str) -> Vec<f64>
pub fn read_field(&self, name: &str) -> Vec<f64>
Reads a field from the model.
Sourcepub fn save_int_field(&mut self, name: &str, data: &[i64])
pub fn save_int_field(&mut self, name: &str, data: &[i64])
Attaches an integer field for serialization in the checkpoint.
Sourcepub fn load_int_field(&self, name: &str, data: &mut Vec<i64>)
pub fn load_int_field(&self, name: &str, data: &mut Vec<i64>)
Reads an integer field from the checkpoint.
Sourcepub fn save_meta(&mut self, name: &str, data: &str)
pub fn save_meta(&mut self, name: &str, data: &str)
Saves meta data associated with a given string.
Sourcepub fn load_meta(&self, name: &str, data: &mut String)
pub fn load_meta(&self, name: &str, data: &mut String)
Loads meta data associated with a given string into data
Sourcepub fn write_meta<T: ToString>(&mut self, name: &str, data: T)
pub fn write_meta<T: ToString>(&mut self, name: &str, data: T)
Writes meta data associated with a given type to the checkpoint.
Sourcepub fn read_meta<T: FromStr>(
&self,
name: &str,
) -> Result<T, CheckpointParseError>
pub fn read_meta<T: FromStr>( &self, name: &str, ) -> Result<T, CheckpointParseError>
Reads meta data associated with a given type from the checkpoint.
Sourcepub fn import_dat(path: impl AsRef<Path>) -> Result<Self>
pub fn import_dat(path: impl AsRef<Path>) -> Result<Self>
Loads the mesh and any additional data from disk.
Sourceimpl<const N: usize> Checkpoint<N>
impl<const N: usize> Checkpoint<N>
pub fn export_csv( &self, path: impl AsRef<Path>, stride: ExportStride, ) -> Result<()>
Sourcepub fn export_vtu(
&self,
path: impl AsRef<Path>,
config: ExportVtuConfig,
) -> Result<()>
pub fn export_vtu( &self, path: impl AsRef<Path>, config: ExportVtuConfig, ) -> Result<()>
Checkpoint and additional field data to a .vtu file, for visualisation in applications like Paraview. This requires a mesh be attached to the checkpoint.