pub trait Projection<const N: usize> {
// Required method
fn project(&self, position: [f64; N]) -> f64;
}Expand description
A projection takes in a position and returns a system of values.
pub trait Projection<const N: usize> {
// Required method
fn project(&self, position: [f64; N]) -> f64;
}A projection takes in a position and returns a system of values.