Module solver

Module solver 

High-level solvers and integrators for common numerical problems.

Structs

HyperRelaxSolver
A solver which implements the algorithm described in NRPyElliptic. This transforms the elliptic equation 𝓛{u} = p, into the hyperbolic equation ∂ₜ²u + η∂ₜu = c² (𝓛{u} - p), where c is the speed of the wave, and η is a dampening term that speeds up convergence.
Integrator

Enums

HyperRelaxError
Error which may be thrown during hyperbolic relaxation.
Method
Method to be used for numerical intergration of ODE.

Traits

SolverCallback
Trait for implementing solver callbacks (most often used to output visualization data).