pub struct HyperRelaxSolver {
pub tolerance: f64,
pub max_steps: usize,
pub dampening: f64,
pub cfl: f64,
pub adaptive: bool,
/* private fields */
}Expand description
A solver which implements the algorithm described in NRPyElliptic. This transforms the elliptic equation ๐{u} = p, into the hyperbolic equation โโยฒu + ฮทโโu = cยฒ (๐{u} - p), where c is the speed of the wave, and ฮท is a dampening term that speeds up convergence.
Fields
tolerance: f64Error tolerance (relaxation stops once error goes below this value).
max_steps: usizeMaximum number of relaxation steps to perform
dampening: f64Dampening term ฮท.
cfl: f64CFL factor for ficticuous time step.
adaptive: boolIf set, the relax solver uses larger time steps for vertices in less refined regions (subject to the CFL condition of course).
Implementations
Sourceimpl HyperRelaxSolver
impl HyperRelaxSolver
Sourcepub fn solve<const N: usize, C: SystemBoundaryConds<N> + Sync, F: Function<N> + Sync>(
&mut self,
mesh: &mut Mesh<N>,
order: usize,
conditions: C,
deriv: F,
result: ImageMut<'_>,
) -> Result<(), HyperRelaxError<F::Error, Infallible>>
pub fn solve<const N: usize, C: SystemBoundaryConds<N> + Sync, F: Function<N> + Sync>( &mut self, mesh: &mut Mesh<N>, order: usize, conditions: C, deriv: F, result: ImageMut<'_>, ) -> Result<(), HyperRelaxError<F::Error, Infallible>>
Solves a given elliptic system
pub fn solve_with_callback<const N: usize, C: SystemBoundaryConds<N> + Sync, F: Function<N> + Sync, Call: SolverCallback<N>>( &mut self, mesh: &mut Mesh<N>, order: usize, conditions: C, callback: Call, deriv: F, result: ImageMut<'_>, ) -> Result<(), HyperRelaxError<F::Error, Call::Error>>